import { Vue } from "../index";
import Dialog from "../nav/dialog.vue";
import Sheet from "../nav/sheet.vue";
import Snackbar from "../nav/snackbar.vue";
import { createApp, h } from "vue";

export default class {
  static get dialog() {
    return LaunchDialog;
  }

  static get sheet() {
    return LaunchSheet;
  }

  static get snackbar() {
    return LaunchSnackbar;
  }
}

class LaunchDialog {
  static _initStack() {
    if (!this.stack) {
      this.stack = [];
    }
  }

  static topDialog() {
    this._initStack();

    return this.stack.last();
  }

  static open(properties, component) {
    if (!this.stack) {
      this.stack = [];
    }


    // https://css-tricks.com/creating-vue-js-component-instances-programmatically/
    const props = {
      spec: properties,
      stack: this.stack,
      vueApp: window.vueApp
    }

    const instance = createApp(
      {
        render: function () {
          return h(Dialog, props)
        }
      }
    )
    Object.assign(instance._context, Vue._context)

    if (component) {
      const placeholder = document.createElement('div')
      document.body.appendChild(placeholder)
      instance.mount(placeholder);
    } else {
      console.error("A dialog has to be displayed in a component");
    }
  }

  static reload(properties, component) {
    if (!this.stack) {
      return;
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.reload(properties);
    });
  }

  static close(properties, component) {
    if (!this.stack) {
      this.stack = [];
    }

    Utils.type.ifObject(this.stack.last(), dialog => {
      dialog.close();
    });

    Utils.type.ifObject(properties["onClose"], it => {
      Action.execute(it, component);
    });
  }

  // This is only meant to be used internally
  static alert(message, component) {
    const properties = {
      message: message
    };
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "OK",
          onClick: properties.onClose
        }
      ]
    });
    this.open(spec, component);
  }
}

class LaunchSheet {
  static open(properties, component) {
    const instance = createApp(
      {
        render: function () {
          return h(Sheet, { spec: properties })
        }
      }
    )
    Object.assign(instance._context, Vue._context)

    if (component) {
      const placeholder = document.createElement('div')
      document.body.appendChild(placeholder)
      instance.mount(placeholder);
    } else {
      console.error("A sheet has to be displayed in a component");
    }

    return instance;
  }

  static confirm(properties, onConfirm, component) {
    const spec = Object.assign({}, properties, {
      buttons: [
        {
          text: "Confirm",
          onClickFunction: onConfirm
        }
      ]
    });
    this.open(spec, component);
  }

  static alert(message, buttons, component) {
    const spec = { message: message, buttons: buttons };
    this.open(spec, component);
  }
}

class LaunchSnackbar {
  static open(properties, component) {
    const instance = createApp(
      {
        render: function () {
          return h(Snackbar, { spec: properties, vueApp: window.vueApp })
        }
      }
    )
    Object.assign(instance._context, Vue._context)

    if (component) {
      const placeholder = document.createElement('div')
      document.body.appendChild(placeholder)
      instance.mount(placeholder);
      instance.show = true;
    } else {
      console.error("A Snackbar has to be displayed in a component");
    }

    return instance;
  }

  static error(message, component) {
    this.open({ message: message, styleClasses: ["error"] }, component);
  }

  static indicator(message, component) {
    const instance = this.open({ message: message, timeout: -1 }, component);
    instance.indicator = true;
  }
}
